/**
 * Module dependencies
 */

var assign = require('object-assign')
var flatten = require('./flatten')
var compose = require('./compose')
var isFSA = require('./is-fsa')
var slice = require('sliced')
var Tree = require('./tree')
var fold = require('./fold')

/**
 * Export `Alley`
 */

module.exports = Alley

/**
 * Initialize `Alley`
 */

function Alley () {
  var tree = Tree(merge(slice(arguments)))
  var send = Send(tree)
  var on = On(tree)
  var use = Use({
    send: send,
    on: on
  })

  return {
    send: send,
    use: use,
    on: on,
    alley: true,
    tree: tree
  }
}

/**
 * Send a value
 */

function Send (tree) {
  return function send (payload) {
    var action = fold(slice(arguments))
    if (!isFSA(action)) throw nonstandard(action)
    var root = tree.up('root') || []
    var parents = tree.up(action.type) || []
    var fn = compose(flatten(root.concat(parents)))
    return fn(action).then(() => action)
  }
}

/**
 * On a function
 */

function On (tree) {
  return function on (name, fn) {
    if (arguments.length === 1) {
      fn = name
      name = 'root'
    }

    let fns = tree(name) || []
    fns = fns.concat({
      fn: fn,
      state: {
        name: name
      }
    })
    tree(name, fns)
  }
}

/**
 * Use is for plugins
 */

function Use (api) {
  function use (fn) { fn(api) }
  api.use = use
  return use
}

/**
 * Merge the incoming data
 */

function merge (alleys) {
  return alleys.reduce(function (data, alley) {
    return assign(data, alley.tree.data)
  }, {})
}

/**
 * Non-standard
 *
 * @param {Mixed} action
 * @return {Error}
 */

function nonstandard (action) {
  return new Error('resolved action (' + JSON.stringify(action) + ') is not the correct format. Please refer to the spec: https://github.com/acdlite/flux-standard-action#actions')
}
